function [opt] = Brlskps(min_m1, min_m2, step, M, X, y, h, lam1,lam2)

% kps: kron parameters selection, here using BRLS algorithm
% Given m1,m2 Minimum value and growth step (i.e., loop different m1,m2)
% P depends on m2 and subject to  floor( M/(m1+m2) ) 

% OutPut the optimal kron parameters {m1,m2,P}_{opt}

%% m1,m2 constraint case
% min_m2 = 5;         % m2>=5
m1 = min_m1;          % m1>=10 
jishu = 1;

mP=[];                                  % All m1,m2,P conbination
while m1 < M / min_m2 
  
    m2 = floor( M / m1 );
    if m1>m2
        
        m1set(jishu,:) = m1;   
        m2set(jishu,:) = m2;      
        
        c=[];
        for p=1: floor( M/(m1+m2) )
            a(p,jishu) = (m1+m2)*p;                                                                         % independent variable \Psi
            b(p,jishu) = BrlsMse(m1, m2, p, X, y, h, lam1,lam2);                              % dependent variable MSE(y)
            c(:,p)=[m1;m2;p];                   
        end
        mP = [mP,c];                              % All parameters conbination
         jishu = jishu+1
         m1 = min_m1+(jishu-1)*step;           % step =5
    else
         jishu = jishu+1
         m1 = min_m1+(jishu-1)*step;
         
    end

end

aa = a;
aa(find(aa==0))=[];
[Psi,ind] = sort(aa);

bb = b;
bb(find(bb==0))=[];
mse = bb(:,ind);

mP = mP(:,ind);

%% BIC evalute
N = length(y);    % one pulse data sample
z1 = -N*log( abs(mse) ) ; 
z2 = Psi*log(N);
BIC = z1 + z2;  %BIC curve

[~,qc] = find(BIC==min(BIC))
opt = [mP(:,qc);mse(:,qc)];     % optimal kron parameters {m1,m2,P}_{opt}

figure(1)
plot(Psi,BIC,'--*');xlabel('\Psi');title(['GBRLS Kron Parameters Selection']);
figure(2)
subplot(3,1,1);plot(Psi,z1,'--o');xlabel('\Psi');title('z1=L*ln( \eta(\Psi) )');
subplot(3,1,2);plot(Psi,z2,'--+');xlabel('\Psi');title('z2=\Psi*ln(L)');
subplot(3,1,3);plot(Psi,BIC,'--*');xlabel('\Psi');title('BIC');
end





















     %% Function Section
function USQ_2 = UnderScore2(X, h_2,m1, m2, P)
      
%      [m2,~] = size(h_2);
%      m1 = size(X,1) / m2;
ind = m1*m2;
     
     QP=[];
         for p=1:P
             Qp =  X(1:ind,:).'*kron(eye(m1),h_2(:,p)) ;
             QP = [QP,Qp];
         end
         USQ_2 = QP;
             
end

function USQ_1 = UnderScore1(X, h_1, m1, m2, P)
      
%      [m1,~] = size(h_1);
%      m2 = size(X,1) / m1;
ind = m1*m2;
     
     QP=[];
         for p=1:P
             Qp =  X(1:ind,:).'*kron(h_1(:,p),eye(m2)) ;
             QP = [QP,Qp];
         end
         USQ_1 = QP;
             
end

function C = kronSum( A,B,P )

% return the kronecker sum of two matrix

C=[];
for p = 1:P
    
    c = kron(A(:,p),B(:,p));
    C = [C,c];
    
end
     C = sum(C,2);
end