% FUNCTION [ynew , tau] = DriftEstimator(x, y, [M], fs)
%
% The function calibrates the y according to x as a benchmark,  where estimates time-delay between the signal x and y using an
% interpolation factor of M (default 1). The implementation follows 
%
% S. L. Marple, "Estimating group delay and phase delay via discrete-time
% "analytic" cross-correlation", IEEE Transactions of Signal Processing,
% Vol. 47, No. 9, pp. 2604-2607, Sept. 1999.
%

function [ynew , tau] = DriftEstimator(x, y, fs)

N = length(x);
ff = ( ( (1:N)'/N -.5 ) );
Ts = 1/fs;

S1 = fft(x, N);
S2 = fft(y, N); 

tau = estTimeDelay( x, y, 1, fs ); %From Andreas

ynew = real( ifft( S2.*exp(1i*2*pi*(ff*fs) * tau ) ,N )  );    %\tilde{st},  compensate y, i.e. obtain calibrated ynew
end