function Cz = DriftEstimator_Xcorr( T )

% This function is used to calibrated/corrected a set of detection variable 'T' result from moving target using correlation peak technique, 
%
% T: detection variable, vector/matrix
% Cz: Corrected detection variable
%
% Consider a pairwise pulses time-delay and calibrate all pulses
% according to benchmark. 
% Ex: seen in the DemoDriftEstimator_Xcorr.m 
%
% Note that there is another Fourier Version named: DriftEstimator.m

%% Estimate time-delay using Xcorr Version, 
fs = 16e3;
[N,K] = size(T);

if  K==1 % T is no need to be corrected
    Cz = T;
else
            
        for t =1:K-1
            [ r(:,t),lags ] = xcorr( T(:,t),T(:,t+1) );
            t_g1(t)     =  find( r(:,t)==max( r(:,t) ) );
            T_g1(t) = ( t_g1(t) - N ) / fs ;%time-delay per pulse
        end

        R = sum( r,2 );
        t_g = find( R==max(R) );
        %% Calibrated 

        for t =1:K
            index = N - (K-t)*abs( N - t_g );    
            Cz(:,t) = [T( index+1:N , t ) ; T( 1:index , t )]; % calibrated z
        end
        
end

end    
