function Hhat = Estimator_Exp( Y, X, s, Y0 )



%% Estimator
%@ Note that we use mutiple estimators to tracking the CIRs
%@ especially the KBSCG has a structure parameters depends on M (i.e., channel dimension)
[L,PulseNum] = size(Y);
N = length(s);
M = L-N;

lambda = 0.7;
%% Setting
            %% BRLS Settings
            h_rls = zeros(M,PulseNum);
            P_n = eye(M);
            %% auto l1-BRLS Settings
            rho1 = M/3;%round (norm(H(:,1)/max(abs(H(:,1))),1) );

            h_l1rls = zeros(M,PulseNum);  % l1-BRLS
            htmp = zeros(M,PulseNum); 
            P_nl1 = eye(M);                
            %% mNLMS Settings
             delta = 1e-6;
             h_mnlms = zeros(M,PulseNum);

             R =  eye(M) ; 
             b = zeros(M,1); 
             p = zeros(M,1); 
            %% IPNLMS Settings  
            h_ipn = zeros(M,PulseNum); 
            sigma = 0.99;
            delta = 1e-6;
            mu = 0.7;
            %% BSCG Settings
            xi = .7;    %sparse factor
            delta = 1e-6;
            c = 1e-4;

            h_scg = zeros(M,PulseNum); 
            Rn = eye(M) ; 
            bn = zeros(M,1); 
            pn = zeros(M,1); 
            %% KBSCG Settings
            m1 = 65;    m2 = 12;      P = 5;
            h1_scg = zeros(m1*P,PulseNum); 
            h2_scg = zeros(m2*P,PulseNum); 
            Rn1 = eye(P*m1) ; Rn2 = eye(P*m2) ; 
            bn1 = zeros(P*m1,1); bn2 = zeros(P*m2,1); 
            pn1 = zeros(P*m1,1); pn2 = zeros(P*m2,1); 

            Original_h2scg = rand(m2,P);   %initialization h_{2}
            Q_2scg = UnderScore2(X, Original_h2scg, m1, m2, P);
            %% BCE Settings
            w_c = zeros(M,PulseNum);
            P_c = eye(M);

            h_c = zeros(M,PulseNum); 
            Rnc = eye(M) ; 
            bnc = zeros(M,1); 
            pnc = zeros(M,1); 

            h1_c = zeros(m1*P,PulseNum); 
            h2_c = zeros(m2*P,PulseNum); 
            Rnc1 = eye(P*m1) ; Rnc2 = eye(P*m2) ; 
            bnc1 = zeros(P*m1,1); bnc2 = zeros(P*m2,1); 
            pnc1 = zeros(P*m1,1); pnc2 = zeros(P*m2,1); 
            Original_h2c = rand(m2,P);   %initialization h_{2}
            Q_2c = UnderScore2(X, Original_h2c, m1, m2, P);

            h_ipn_c = zeros(M,PulseNum); 
%% Loop
tic
for t = 1:PulseNum  
            %% BRLS loop
            [h_rls(:,t+1),P_n] = RLSupdate(X, Y(:,t), h_rls(:,t), P_n, lambda);
            %% auto l1-BRLS loop                  
            [h_l1rls(:,t+1), htmp(:,t+1), P_nl1, gml1(t)] = AutoL1_RLSupdate(X, Y(:,t), h_l1rls(:,t), htmp(:,t), P_nl1, lambda, rho1);    
            %% IPNLMS loop                
            [h_ipn(:,t+1)] = IPNLMSupdate(X, Y(:,t), mu, h_ipn(:,t), delta, sigma); 
            %% mNLMS loop      
            [ h_mnlms(:,t+1), R, b, p ] = mNLMSupdate( lambda, delta, X, Y(:,t), h_mnlms(:,t), R, b, p );                     
            %% BSCG loop                 
            [h_scg(:,t+1),Rn,bn,pn] = SCGupdate(lambda, xi, delta, c, X, Y(:,t), h_scg(:,t), Rn, bn, pn);
            %% KBSCG loop                 
            [h_scgkron(:,t), h1_scg(:,t+1), h2_scg(:,t+1), Rn1, Rn2, bn1, bn2, pn1, pn2, Q_2scg] = KronSCGupdate(X, Q_2scg, Y(:,t), h1_scg(:,t), h2_scg(:,t), Rn1, Rn2, bn1, bn2, pn1, pn2, lambda, lambda, xi, delta, c, m1, m2, P) ;   
            %% BCE loop
            [h_C(:,t), w_c(:,t+1), P_c,...
                 h_c(:,t+1), Rnc, bnc, pnc,...
                 h1_c(:,t+1), h2_c(:,t+1), Rnc1, Rnc2, bnc1, bnc2, pnc1, pnc2, Q_2c,...
                 h_ipn_c(:,t+1)] =...
                                             IPExpBCEupdate(lambda, xi, delta, c,...
                                               w_c(:,t), P_c,...
                                               h_c(:,t), Rnc, bnc, pnc,...
                                               X, Q_2c, Y(:,t), Y0(:,t), ...
                                               h1_c(:,t), h2_c(:,t), Rnc1, Rnc2, bnc1, bnc2, pnc1, pnc2, lambda, lambda, m1, m2, P,...
                                               h_ipn_c(:,t),mu,sigma);  
                                   
end             
%% Hhat
h_rls          = h_rls(:,2:end);
h_l1rls       = h_l1rls(:,2:end);
h_ipn         = h_ipn(:,2:end);
h_mnlms   = h_mnlms(:,2:end);
h_scg        = h_scg(:,2:end);
h_scgkron = h_scgkron;
h_C            = h_C;
Esttoc = toc
%% Check Est Performance Eta
Hhat = [{h_rls}, {h_l1rls}, {h_ipn}, {h_mnlms}, {h_scg}, {h_scgkron}, {h_C}];
ExpPlotEstPerformance( Y0, X, Hhat );

end




%% Function Section
function USQ_2 = UnderScore2(X, h_2,m1, m2, P)
      
%      [m2,~] = size(h_2);
%      m1 = size(X,1) / m2;
ind = m1*m2;
     
     QP=[];
         for p=1:P
             Qp =  X(1:ind,:).'*kron(eye(m1),h_2(:,p)) ;
             QP = [QP,Qp];
         end
         USQ_2 = QP;
             
end

function USQ_1 = UnderScore1(X, h_1, m1, m2, P)
      
%      [m1,~] = size(h_1);
%      m2 = size(X,1) / m1;
ind = m1*m2;
     
     QP=[];
         for p=1:P
             Qp =  X(1:ind,:).'*kron(h_1(:,p),eye(m2)) ;
             QP = [QP,Qp];
         end
         USQ_1 = QP;
             
end

function C = kronSum( A,B,P )

% return the kronecker sum of two matrix

C=[];
for p = 1:P
    
    c = kron(A(:,p),B(:,p));
    C = [C,c];
    
end
     C = sum(C,2);
end