function outputT  = ExpAMFD( Y, X, Hhat, initialK, s )

% AMFD: Augmented Median Filter Detector
% initialK: target occuring pulse
% Y: measurement
% Hhat: cell-matrix including multiple estimators

fs = 16e3;

%% No consider CIR matchfilter
%
y = Y(:,initialK:end);                                          %measurement matrix, y_k
y = prewhitening( y );                                      %whitening process
mfT = formDetectionVariable1( y, s );             %detection variable matrix,  T_k   
outputmfT = max( sum(mfT,2) );
yk   = mean( y ,2 );                                           %save y_k
%

%% Consider CIR matchfilter
[~, EstorNum] = size(Hhat);
for n = 1:EstorNum
     % Residual Process   e(k) = y(k) - X*h(k-1)    

            e       = Y(:,initialK:end) - X'*Hhat{n}(:,initialK-1:end-1);   %residual matrix, for caseB: moving target being present in all pulse       (initialK > 2)
            e       = prewhitening( e );                                                %whitening process
            T       = formDetectionVariable1( e, s );                           %detection variable matrix, T  
            cT     = DriftEstimator_Xcorr( T );                                     %correct detection variable, \tildeT
            bT     = sum( cT,2 );                                                          %improved detection variable, \breveT
            outputestT(n) = max(bT);

            % Save for Plot
            ek(:,n)    = mean( e,2 );                                                 %save mean residual of estimators
            E{n}       = e;                                                                  %save residual of estiamtors
            estT{n}  = T;                                                                  %save detection variable 'T' of estimators
            tT{n}     = cT;                                                                 %save correct detection variable '\tildeT' of estimators
            BT(:,n)  = bT;                                                                 %save improved detection variable '\breveT' of estimators
            
end

% estimators T & mf T output
outputT = [ outputestT, outputmfT ];

%% Check residual  Normal & White
fprintf('There is %d estimators \n',(EstorNum));
fprintf('The sequence is BRLS, autol1-BRLS, IPNLMS, mNLMS, BSCG, KBSCG, BCE \n')

checkIfNormal( ek, 'residual' )
checkIfWhite( ek )
checkIfWhite( yk )

%%
% # plot \Breve{T}_K
[N,~] = size( T );

figure;
plot( (1:N)/fs, BT, '--.'); hold on;  
plot( (1:N)/fs, sum( mfT , 2 ),  '--*', 'LineWidth',1 );        % Plot sum(T(yk))
legend('BRLS','autol1-BRLS','IPNLMS','mNLMS','BSCG','KBSCG','BCE','UMF');
xlabel('Time(s)');ylabel('$\breve{T}_K$','interpreter','latex');
% title('$Calibrated \sum T(k)$, where e(k)= y(k) - X*h(k-1)','interpreter','latex');    %ylim([0 30])

% # refineplot T, \tildeT, \breveT

nn =7;  % estimator serial number
figure;
plot( (1:N)/fs , estT{nn}(:,end-2:end), '--.', 'LineWidth',1); hold on;
% plot( (1:N)/fs , estT{nn}(:,[1,3,4]), '--.', 'LineWidth',1); hold on;
legend('$T_{8}$','$T_{9}$','$T_{10}$','interpreter','latex');
xlabel('Time(s)');ylabel('$T_k$','interpreter','latex');axis tight;
% title('T(k), e(k)= y(k) - X*h(k-1)','interpreter','latex')

figure;
plot( (1:N)/fs , tT{nn}(:,end-2:end) , '--.', 'LineWidth',1);  hold on;
% plot( (1:N)/fs , tT{nn}(:,[1,3,4]) , '--.', 'LineWidth',1);  hold on;
plot( (1:N)/fs , BT(:,nn ) , 'LineWidth',1.5);hold on;
legend('$\tilde{T}_{8}$','$\tilde{T}_{9}$','$\tilde{T}_{10}$','$\breve{T}_K$','interpreter','latex');
xlabel('Time(s)');ylabel('$\tilde{T}_k$','interpreter','latex'); axis tight;
% title('Calibrated T(k), e(k)= y(k) - X*h(k-1)','interpreter','latex')

fprintf('ExpAMFD ( Augmented Median Filter Detector ) function is over  \n')
end














%%
%%
%%
% e1 = Y(:,initialK:end) - X'*Hhat{n}(:,initialK:end);              %e(k) = y(k) - X*h(k)
% e1 =  prewhitening( e1 );                                                 %whitening process
% e1T = formDetectionVariable1( e1, s );                            %matrix  T(ek), where e(k) = y(k)-X*h(k)
% % Save for Plot
% e1k(:,n) = mean( e1,2 );                                                   %Save residual of estimators
% estT1{n} = e1T;                                                                %Save T( e(k) ) of estimators, where e(k) = y(k)-X*h(k)
%

% e2 = Y(:,initialK-1:end-1) - X'*Hhat{n}(:,initialK-2:end-2);     %e(k-1) = y(k-1)-X*h(k-2)
% e2 =  prewhitening( e2 );
% e2T = formDetectionVariable1( e2, s );                          %matrix  T( e(k-1) ), where e(k-1) = y(k-1)-X*h(k-2)
% dT = abs(eT - e2T);                                                         %matrix  \DeltaT( k ) = T( e(k) ) - T( e(k-1) )

% E(:,n) = mean( e,2 );                        %Save residual of estimators
% estT{n} = eT;                                   %Save T( e(k) ) of estimators, where e(k) = y(k)-X*h(k-1)
% estT1{n} = e1T;                               %Save T( e(k) ) of estimators, where e(k) = y(k)-X*h(k)
% estT2{n} = dT;                                 %Save \DeltaT( e(k) ) of estimators, where e(k) = y(k)-X*h(k-1)
%%

% z = eT;
% [N,~] = size( z );
%% Calibrate T(ek) & T(yk) using the Xcorr Version
% Cz = DriftEstimator_Xcorr( z );
%%
%%
% Calibrate T(ek) & T(yk) using the Fourier Version
% for t = 1:B    
% [cali_eT(:,t), eTtau(:,t)]    =  DriftEstimator(eT(:,end), eT(:,t), 16e3);
% [cali_mfT(:,t), mftau(:,t)] = DriftEstimator( mfT(:,end), mfT(:,t), 16e3 );
% % [cali_dT(:,t), dTtau(:,t)]   =  DriftEstimator(dT(:,end), dT(:,t), 16e3);
% end
%%

% estimators T output
% T(n) = max( sum( Cz ,2 ) );          %scaler

% Save for Plot
% CestT{n}   = Cz;                               % Save calibrated T( e(k) ) of estimators, where e(k) = y(k)-X*h(k-1)
