function [h_kron, h1_t, h2_t, Rn1, Rn2, bn1, bn2, pn1, pn2, Q_2] = KronSCGupdate(X, Q_2, y, h1, h2, R1, R2, b1, b2, p1, p2, lam1, lam2, xi, delta, c, m1, m2, P)

% xi: a factor used to control the sparsity
% delta, c:  a small regularization constant,  preventing division by zero
% m1, m2, P: Kron structure parameters
% lam1, lam2: forgetting factors

M = size(X,1);

[h1_t, Rn1, bn1, pn1] = SCGupdate(lam1, xi, delta, c, Q_2.', y, h1, R1, b1, p1);
h1P = reshape(h1_t,[m1,P]);  
Q_1 = UnderScore1(X, h1P, m1, m2, P);
[h2_t, Rn2, bn2, pn2] = SCGupdate(lam2, xi, delta, c, Q_1.', y, h2, R2, b2, p2);
h2P =  reshape(h2_t,[m2,P]);  
Q_2 = UnderScore2(X, h2P, m1, m2, P);

 h_kron = zeros(M,1); 
 ind =[1: m1*m2];
 h_kron(ind) = kronSum( h1P,h2P,P );
 
end



%% Function Section
function USQ_2 = UnderScore2(X, h_2,m1, m2, P)
      
%      [m2,~] = size(h_2);
%      m1 = size(X,1) / m2;
ind = m1*m2;
     
     QP=[];
         for p=1:P
             Qp =  X(1:ind,:).'*kron(eye(m1),h_2(:,p)) ;
             QP = [QP,Qp];
         end
         USQ_2 = QP;
             
end

function USQ_1 = UnderScore1(X, h_1, m1, m2, P)
      
%      [m1,~] = size(h_1);
%      m2 = size(X,1) / m1;
ind = m1*m2;
     
     QP=[];
         for p=1:P
             Qp =  X(1:ind,:).'*kron(h_1(:,p),eye(m2)) ;
             QP = [QP,Qp];
         end
         USQ_1 = QP;
             
end

function C = kronSum( A,B,P )

% return the kronecker sum of two matrix

C=[];
for p = 1:P
    
    c = kron(A(:,p),B(:,p));
    C = [C,c];
    
end
     C = sum(C,2);
end