function [hn, Rn, bn, pn] = SCGupdate(lambda, gamma, epslion, c, un, dn, h0, R0, b0, p0)

%  Input:
% lambda:= is the forgetting factor
% gamma:= is the sparse factor, i.e. p is smaller and sparser
% epslion:= is for preventing division by zero.
% c:= is relatively small as compared to the estimate, so it would not affect the convergency

% un:= is input vector
% dn:= is the data sample
% R0:= is the correlation matrix
% b0:= is the cross-correlation vector
% p0:= is the search direction

% Output:
% hn,Rn,bn,pn


%Update
Rn = lambda*R0 + un*un.';
bn = lambda*b0 + un*dn;

%% formulation (14) norm2
% Wn = diag(   sqrt(   2/gamma * (abs(h0) + c).^(2-gamma)  )  );
%% formulation (15)  norm1
Wn = diag(  ( abs(h0) + c ).^(1-gamma) / gamma  );

gn = Wn * (Rn*h0 - bn);

Nom1 = p0.'*Wn*Rn*Wn*p0 + epslion;
betan = p0.'*Wn*Rn*Wn*gn / Nom1;
pn = -gn + p0*betan;
Nom2 = pn.'*Wn*Rn*Wn*pn + epslion;
alphan = - pn.'*gn / Nom2;

hn = h0 + Wn*pn*alphan;
end
