% FUNCTION t_g = estTimeDelay( x, y, [M] )
%
% The function estimates the time-delay between the signal x and y using an
% interpolation factor of M (default 1). The implementation follows
%
% S. L. Marple, "Estimating group delay and phase delay via discrete-time
% "analytic" cross-correlation", IEEE Transactions of Signal Processing,
% Vol. 47, No. 9, pp. 2604-2607, Sept. 1999.
%
function t_g = estTimeDelay( x, y, M, fs )

if nargin<3
    M=1;
end

x = x(:);
y = y(:);
N = length(x);

X = fft( [x;zeros(N,1)] );
Y = fft( [y;zeros(N,1)] );
C_yx = Y.*conj(X);
c_yx = fftshift( ifft(C_yx));

C_yx_interp_anal = [C_yx(1) ; 2*C_yx(2:N) ; C_yx(N+1) ; zeros(2*M*N-N-1,1) ];
c_yx_interp_anal = fftshift( ifft(C_yx_interp_anal))*M;

[~,m] = max( abs(c_yx_interp_anal) );

Ts = 1/fs;
t_g = (m-N*M-1)/M * Ts;

end