function [targetS, TarResp]= formComplexTargetSignal ( Y, s, fs, option, K )

% The experiment scanrios as follows: 
% the distance between transmitter and receiver  is 5km,
% a pulse repeat time (PRT) is 5km/c = 1/3 (s), in every CIR response the main
% target response moving a 'delta_tau' time-delay
%
% option = 1, we set a complex target response
% option = 0, only main target response
%
% K: added target pulseNmuber, which is used to improve detection performance
%
% Default: delta_tau=3ms, v=18, mtau=0.025s, 3 responses with randomly amplitude  [for ExpMC]


c=1500; %Sound speed
% PRT = 5e3/c;
v = 18;
% delta_tau = PRT*v / c; 
delta_tau = 3e-3;

% random target scattering response:
% Main path is propagated according to the constant velocity,
% the weak paths resut from surface or bottom are random

% note that v> p*c / (fs*PRT), 


[L,~] = size( Y );
for i=1:K

    
targetResponse = zeros(L-length(s),1);  %M*PulseNum

mtau =0.033+(i-1)*delta_tau;  %target main response time-delay in per pulse
p = 5; %target response length
mtaps = floor(mtau*fs)          
wtaps=mtaps + linspace(1,p,p);
mamp = 1;                            
wamp = option*0.5*rand( size(wtaps) ); %weak amplitudes are 10 times less than main amplitude
targetResponse( [mtaps,wtaps] )  = [mamp,wamp];

TarResp(:,i) = targetResponse;

% conv transmit signal and target response = echo
% Doppler effect results from the target motion, using resample rate to 
% illustrate the doppler effect


DoppFactor  = 2*v / c;
targetsignal = conv( s, targetResponse );
targetsignal1 = targetsignal( targetsignal ~= 0 );% Remove zeros
echo= [zeros(floor(mtau*fs),1); resample( targetsignal1 , round( ( 1+DoppFactor  )*1e4 ) , 1e4) ];
targetS(:,i) = [echo;zeros(L-length(echo),1)];

 
end


end