function medT = formDetectionVariable1( e, s )

% This function is used to generate detection variable 'T' by sliding window
% matched filtering technology based on 'residual'.
%
% e: residual matrix/vector
% s: transmitte signal, used for matchfilter
%
% The sliding window width 'P' in medfilt1(T,window) suffers from noise,
% which means  larger/wider windows can indeed reduce the impact of noise,
% further more effects the detection performance of ROC.
%
% In the simulation, we use P=50 to adapt low SNR case.
%
% Default: P=100 [for ExpMC]
%

fs = 16e3;
[L,num] = size(e);
N = length(s);
s0 = s(:);

for i = 1: num

        for k = 1:L-N+1
            sv = e(k:k+N-1,i);
            b = sv' * s0; %project onto signal subspace
            q = b'*b / (s0'*s0);
            p = sv'*sv - q;
            T(k) = (N-1) * q / p;
        end

medT (:,i) = medfilt1(T,100);   %using Median Filtering

end

% figure;
% gamma = finv(0.95, 1, L); %threshold
% plot( (1:k)/fs , [T' sum(medT,2) gamma*ones(k,1)]); %show the target location
% legend('$\psi(\hat{\bf n}_k(\tau))$', '$T_{k}$','$\gamma_\psi$' ,'Location', 'NW', 'interpreter','latex' )  
% xlabel('Time');ylabel('$T_k$','interpreter','latex'); axis tight
% ylim([0 30])

 end