% Here considering the drift (pulse) + amp changing (part), mimic the real ocean data.  
% Compare methods performance without/with compensated
% MSD and eta

% Note that \tau= Ts * grid, which \tau is not required integer, but the
% grid must be integer, otherwise drift-delay estimator counld not work
% because of the FFT resolution.

% Note that we denote partand "pulse", we assume in the part there is a
% time-invariant processing, adjacent part existing a time-varying (i.e., concatenated two different channels)

% Output the 'Table1' in the paper


clear all; close all;  clc;

addpath('./functions','./data');

SNR = 0;
Part = 2;% time-varying part
partnum = 40;% CIRs in one part, and all CIRs are time-invariant
PulseNum = partnum*Part;
%% Transmit Signal
Tp = 0.2;   fs = 16e3;  f1 = 3e3; f2 = 7e3;  
s = transmitLFM(Tp,fs,f1,f2);
Ts = 1/fs;
%% Measurment Section with Drifting Channel

% load 'spah.mat';     % sparse CIRs
load 'P27_0707'; z=h0707;    % h is a vector, i.e., h_{initial}


M = length(z)

w = envelope ( z );   % amp weighting coeff
tau = 2*floor( cumsum( rand(PulseNum,1) ) ) * Ts;   %drift time-delay in one part   

for part = 1 : Part
    
    Delta_h  =  w.*randn(M,1)*20; % same weighting in one part    

    for  t = partnum*(part-1)+1 : partnum*part
    
    h(:,t) = TimeDelay(z+ Delta_h,tau(t),fs) ;
    [ y(:,t) , X ] = simulatePulseData_toeplitz(s, h(:,t), 1, SNR);

    end

end
%%%%%%%%
figure;
plot(h(:,partnum-1)); hold on;
plot(h(:,2*partnum-1));hold on
legend('$h_{part1}$','$h_{part2}$','interpreter','latex')
%%%%%%%%%%
%% Compensate Section
            for  t=1:PulseNum
                 [ cali_y(:,t), est_tau(t) ] = DriftEstimator( y(:,1), y(:,t), fs );
                 [ cali_h(:,t), ~ ] = DriftEstimator( h(:,1), h(:,t), fs );
            end
            %%%%%%%%
            figure;
            imagesc(abs(h(1:200,:))); title(['Drift h '])
            figure;
            imagesc(abs(cali_h(1:200,:))); title(['Compansate h'])
            %%%%%%%%
 %% Estimatie Section  
 lambda = .7;  %forgetting factor   
%% BRLS Settings
h_rls = zeros(M,PulseNum);  afh_rls = zeros(M,PulseNum);
P_n1 = eye(M);    P_n2 = eye(M);
%% auto l1-BRLS Settings
rho1 = round (norm(z/max(abs(z)),1) );

h_l1rls = zeros(M,PulseNum);     afh_l1rls = zeros(M,PulseNum); % l1-BRLS
htmp = zeros(M,PulseNum);      afhtmp = zeros(M,PulseNum); 
P_nl1 = eye(M);                          afP_nl1 = eye(M);      
%% mNLMS Settings
 delta = 1e-6;
 h_mnlms = zeros(M,PulseNum);     afh_mnlms = zeros(M,PulseNum);
 
 R =  eye(M) ;         afR =  eye(M) ;  
 b = zeros(M,1);     afb = zeros(M,1); 
 p = zeros(M,1);     afp = zeros(M,1); 
%% IPNLMS Settings
h_ipnlms = zeros(M,PulseNum);   afh_ipnlms = zeros(M,PulseNum); 
sigma = 0.99;
delta = 1e-6;
mu = 0.7;
%% BSCG Settings
xi = 0.7;    %sparse factor
delta = 1e-6;
c = 1e-4;

h_scg = zeros(M,PulseNum);   afh_scg = zeros(M,PulseNum); 
Rn = eye(M) ;  afRn = eye(M) ;
bn = zeros(M,1);   afbn = zeros(M,1);
pn = zeros(M,1);   afpn = zeros(M,1);  
%% KBSCG Settings
m1 = 65;    m2 = 12;      P = 5;
h1_scg = zeros(m1*P,PulseNum); 
h2_scg = zeros(m2*P,PulseNum); 
Rn1 = eye(P*m1) ;           Rn2 = eye(P*m2) ; 
bn1 = zeros(P*m1,1);          bn2 = zeros(P*m2,1); 
pn1 = zeros(P*m1,1);            pn2 = zeros(P*m2,1); 
Original_h2scg = rand(m2,P);   %initialization h_{2}
Q_2scg = UnderScore2(X, Original_h2scg, m1, m2, P);

afh1_scg = zeros(m1*P,PulseNum); 
afh2_scg = zeros(m2*P,PulseNum); 
afRn1 = eye(P*m1) ;              afRn2 = eye(P*m2) ; 
afbn1 = zeros(P*m1,1);            afbn2 = zeros(P*m2,1); 
afpn1 = zeros(P*m1,1);               afpn2 = zeros(P*m2,1); 
afOriginal_h2scg = rand(m2,P);   %initialization h_{2}
afQ_2scg = UnderScore2(X, afOriginal_h2scg, m1, m2, P);
%% BCE Settings
w_c = zeros(M,PulseNum);      afw_c = zeros(M,PulseNum);
P_c = eye(M);                           afP_c = eye(M);

h_c = zeros(M,PulseNum);      afh_c = zeros(M,PulseNum); 
Rnc = eye(M) ;                         afRnc = eye(M) ; 
bnc = zeros(M,1);                    afbnc = zeros(M,1); 
pnc = zeros(M,1);                    afpnc = zeros(M,1); 

h1_c = zeros(m1*P,PulseNum);             afh1_c = zeros(m1*P,PulseNum); 
h2_c = zeros(m2*P,PulseNum);             afh2_c = zeros(m2*P,PulseNum); 
Rnc1 = eye(P*m1) ; Rnc2 = eye(P*m2) ;             afRnc1 = eye(P*m1) ; afRnc2 = eye(P*m2) ; 
bnc1 = zeros(P*m1,1); bnc2 = zeros(P*m2,1);    afbnc1 = zeros(P*m1,1); afbnc2 = zeros(P*m2,1); 
pnc1 = zeros(P*m1,1); pnc2 = zeros(P*m2,1);    afpnc1 = zeros(P*m1,1); afpnc2 = zeros(P*m2,1);
Original_h2c = rand(m2,P);                                            afOriginal_h2c = rand(m2,P);   %initialization h_{2}
Q_2c = UnderScore2(X, Original_h2c, m1, m2, P);         afQ_2c = UnderScore2(X, afOriginal_h2c, m1, m2, P);

h_ipn_c = zeros(M,PulseNum);          afh_ipn_c = zeros(M,PulseNum); 
%% LOOP
for t=1:PulseNum
     y1 = y(:,t) ;  %drift data
     y2 = cali_y(:,t);  %non-drift data
     %% LS loop
     h_ls(:,t) = inv(X*X') * X*y1;
     afh_ls(:,t) = inv(X*X') * X*y2;
     %% BRLS loop
     [h_rls(:,t+1),P_n1] = RLSupdate(X, y1, h_rls(:,t), P_n1, lambda);
     [afh_rls(:,t+1),P_n2] = RLSupdate(X, y2, afh_rls(:,t), P_n2, lambda);
     %% auto l1-BRLS loop
      [h_l1rls(:,t+1), htmp(:,t+1), P_nl1, gml1(t)] = AutoL1_RLSupdate(X, y1, h_l1rls(:,t), htmp(:,t), P_nl1, lambda, rho1);
      [afh_l1rls(:,t+1), afhtmp(:,t+1), afP_nl1, afgml1(t)] = AutoL1_RLSupdate(X, y2, afh_l1rls(:,t), afhtmp(:,t), afP_nl1, lambda, rho1);
     %% mNLMS loop
      [ h_mnlms(:,t+1), R, b, p ] = mNLMSupdate( lambda, delta, X, y1, h_mnlms(:,t), R, b, p );
      [ afh_mnlms(:,t+1), afR, afb, afp ] = mNLMSupdate( lambda, delta, X, y2, afh_mnlms(:,t), afR, afb, afp ); 
     %% IPNLMS loop
      [h_ipnlms(:,t+1)] = IPNLMSupdate(X, y1, mu, h_ipnlms(:,t), delta, sigma); 
      [afh_ipnlms(:,t+1)] = IPNLMSupdate(X, y2, mu, afh_ipnlms(:,t), delta, sigma); 
     %% BSCG loop
     [h_scg(:,t+1), Rn, bn, pn] = SCGupdate(lambda, xi, delta, c, X, y1, h_scg(:,t), Rn, bn, pn);
     [afh_scg(:,t+1), afRn, afbn, afpn] = SCGupdate(lambda, xi, delta, c, X, y2, afh_scg(:,t), afRn, afbn, afpn);     
     %% KBSCG loop
     [h_scgkron(:,t), h1_scg(:,t+1), h2_scg(:,t+1), Rn1, Rn2, bn1, bn2, pn1, pn2, Q_2scg] = KronSCGupdate(X, Q_2scg, y1, h1_scg(:,t), h2_scg(:,t), Rn1, Rn2, bn1, bn2, pn1, pn2, lambda, lambda, xi, delta, c, m1, m2, P) ;
     [afh_scgkron(:,t), afh1_scg(:,t+1), afh2_scg(:,t+1), afRn1, afRn2, afbn1, afbn2, afpn1, afpn2, afQ_2scg] = KronSCGupdate(X, afQ_2scg, y2, afh1_scg(:,t), afh2_scg(:,t), afRn1, afRn2, afbn1, afbn2, afpn1, afpn2, lambda, lambda, xi, delta, c, m1, m2, P) ;
     %% BCE loop
      [h_C(:,t), w_c(:,t+1), P_c,...
          h_c(:,t+1), Rnc, bnc, pnc,...
          h1_c(:,t+1), h2_c(:,t+1), Rnc1, Rnc2, bnc1, bnc2, pnc1, pnc2, Q_2c,...
          h_ipn_c(:,t+1)] =...
                            IPBCEupdate(lambda, xi, delta, c,...
                                                 w_c(:,t), P_c,...
                                                 h_c(:,t), Rnc, bnc, pnc,...
                                                 X, Q_2c, y1, h(:,t), ...
                                                 h1_c(:,t), h2_c(:,t), Rnc1, Rnc2, bnc1, bnc2, pnc1, pnc2, lambda, lambda, m1, m2, P,...
                                                 h_ipn_c(:,t),mu,sigma);
      [afh_C(:,t), afw_c(:,t+1), afP_c,...
          afh_c(:,t+1), afRnc, afbnc, afpnc, afh1_c(:,t+1), afh2_c(:,t+1), afRnc1, afRnc2, afbnc1, afbnc2, afpnc1, afpnc2, afQ_2c,...
          afh_ipn_c(:,t+1)] =...
                            IPBCEupdate(lambda, xi, delta, c,...
                                               afw_c(:,t), afP_c, afh_c(:,t), afRnc, afbnc, afpnc,...
                                               X, afQ_2c, y2, cali_h(:,t), ...
                                               afh1_c(:,t), afh2_c(:,t), afRnc1, afRnc2, afbnc1, afbnc2, afpnc1, afpnc2, lambda, lambda, m1, m2, P,...
                                               afh_ipn_c(:,t),mu,sigma);
                                           
      disp(['pulse=',num2str(t)])
end
%% hEst output
h_rls=h_rls(:,2:end);                      afh_rls=afh_rls(:,2:end);
h_l1rls = h_l1rls(:,2:end);                afh_l1rls = afh_l1rls(:,2:end);
h_mnlms = h_mnlms(:,2:end);         afh_mnlms = afh_mnlms(:,2:end);
h_ipnlms = h_ipnlms(:,2:end);            afh_ipnlms = afh_ipnlms(:,2:end);
h_scg=h_scg(:,2:end);                           afh_scg=afh_scg(:,2:end);
%% Plot  Performance Measure eta and Compute Gain, Normalized

eta_ls = 20*log10( vecnorm( X.'*h_ls - X.'*cali_h ) ./ vecnorm( X.'*cali_h ) );
eta_rls = 20*log10( vecnorm( X.'*h_rls - X.'*cali_h )  ./ vecnorm( X.'*cali_h) );
eta_l1rls = 20*log10( vecnorm( X.'*h_l1rls - X.'*cali_h )  ./ vecnorm( X.'*cali_h) );
eta_ipnlms = 20*log10( vecnorm( X.'*h_ipnlms - X.'*cali_h )  ./ vecnorm( X.'*cali_h) );
eta_mnlms = 20*log10( vecnorm( X.'*h_mnlms - X.'*cali_h )  ./ vecnorm( X.'*cali_h) );
eta_scg = 20*log10( vecnorm( X.'*h_scg - X.'*cali_h  ) ./ vecnorm( X.'*cali_h ) );
eta_kronscg = 20*log10( vecnorm( X.'*h_scgkron - X.'*cali_h )  ./ vecnorm( X.'*cali_h ) );
eta_C = 20*log10( vecnorm( X.'*h_C - X.'*cali_h )  ./ vecnorm( X.'*cali_h ) );

eta_afls = 20*log10( vecnorm( X.'*afh_ls - X.'*cali_h )  ./ vecnorm( X.'*cali_h ) );
eta_afrls = 20*log10( vecnorm( X.'*afh_rls - X.'*cali_h )  ./ vecnorm( X.'*cali_h ) );
eta_afl1rls = 20*log10( vecnorm( X.'*afh_l1rls - X.'*cali_h )  ./ vecnorm( X.'*cali_h) );
eta_afipnlms = 20*log10( vecnorm( X.'*afh_ipnlms - X.'*cali_h )  ./ vecnorm( X.'*cali_h) );
eta_afmnlms = 20*log10( vecnorm( X.'*afh_mnlms - X.'*cali_h )  ./ vecnorm( X.'*cali_h) );
eta_afscg = 20*log10( vecnorm( X.'*afh_scg - X.'*cali_h )  ./ vecnorm( X.'*cali_h ) );
eta_afkronscg = 20*log10( vecnorm( X.'*afh_scgkron - X.'*cali_h )  ./ vecnorm( X.'*cali_h ) );
eta_afC = 20*log10( vecnorm( X.'*afh_C - X.'*cali_h )  ./ vecnorm( X.'*cali_h ) );

figure;
plot(eta_ls,'-o');  hold on;        plot(eta_afls,'--o');  hold on;
plot(eta_rls,'-+');  hold on;      plot(eta_afrls,'--+');  hold on;
plot(eta_l1rls,'-*');  hold on;      plot(eta_afl1rls,'--*');  hold on;
plot(eta_ipnlms,'-.');  hold on;      plot(eta_afipnlms,'--.');  hold on;
plot(eta_mnlms,'-s');  hold on;      plot(eta_afmnlms,'--s');  hold on;
plot(eta_scg,'-*');  hold on;      plot(eta_afscg,'--*');  hold on;
plot(eta_kronscg,'-^');  hold on;       plot(eta_afkronscg,'--^');  hold on;
plot(eta_C,'-d');  hold on;       plot(eta_afC,'--d');  hold on;
legend('LS','com-LS','BRLS','com-BRLS','l1BRLS','com-l1BRLS','IPNLMS','com-IPNLMS','mNLMS','com-mNLMS','BSCG','com-BSCG','KBSCG','com-KBSCG','BCE','com-BCE');
xlabel('Pulse');ylabel('\eta (dB)');title(['SNR=',num2str(SNR),'dB    \eta'])
%% Compute average eta (use steady state)
clearvars Peta_ls Peta_rls Peta_autol1rls Peta_mnlms Peta_ipnlms Peta_scg Peta_scgkron Peta_C 
clearvars Peta_afls Peta_afrls Peta_afautol1rls Peta_afmnlms Peta_afipnlms Peta_afscg Peta_afscgkron Peta_afC 

for part =1: Part  % 
    usepart = (part-1)*partnum+5:part*partnum ; % steady state part
    Peta_ls(part) = mean( eta_ls( :, usepart ) ); % every part average of effcial eta
    Peta_rls(part) = mean( eta_rls( :, usepart ) );
    Peta_l1rls(part) = mean( eta_l1rls( :, usepart ) );
    Peta_mnlms(part) = mean( eta_mnlms( :, usepart ) );
    Peta_ipnlms (part) = mean( eta_ipnlms ( :, usepart ) );
    Peta_scg (part) = mean( eta_scg ( :, usepart ) );
    Peta_kronscg (part) = mean( eta_kronscg ( :, usepart ) );
    Peta_C (part) = mean( eta_C ( :, usepart ) );
end

for part =1: Part  % 
    usepart = (part-1)*partnum+5:part*partnum ; % steady state part
    Peta_afls(part) = mean( eta_afls( :, usepart ) ); % every part average of effcial eta
    Peta_afrls(part) = mean( eta_afrls( :, usepart ) );
    Peta_afl1rls(part) = mean( eta_afl1rls( :, usepart ) );
    Peta_afmnlms(part) = mean( eta_afmnlms( :, usepart ) );
    Peta_afipnlms (part) = mean( eta_afipnlms ( :, usepart ) );
    Peta_afscg (part) = mean( eta_afscg ( :, usepart ) );
    Peta_afkronscg (part) = mean( eta_afkronscg ( :, usepart ) );
    Peta_afC (part) = mean( eta_afC ( :, usepart ) );
end

Meta_ls = mean(Peta_ls);        Meta_afls = mean(Peta_afls);
Meta_rls = mean(Peta_rls);        Meta_afrls = mean(Peta_afrls);
Meta_l1rls = mean(Peta_l1rls);        Meta_afl1rls = mean(Peta_afl1rls);
Meta_ipnlms = mean(Peta_ipnlms);        Meta_afipnlms = mean(Peta_afipnlms);
Meta_mnlms = mean(Peta_mnlms);        Meta_afmnlms = mean(Peta_afmnlms);
Meta_scg = mean(Peta_scg);        Meta_afscg = mean(Peta_afscg);
Meta_kronscg = mean(Peta_kronscg);        Meta_afkronscg = mean(Peta_afkronscg);
Meta_C = mean(Peta_C);        Meta_afC = mean(Peta_afC);

disp(['Output follow averaged eta'])
disp({['LS=',num2str(Meta_ls), 'dB    C-LS=',num2str(Meta_afls), 'dB   Gain=',num2str( abs( Meta_afls-Meta_ls ) ),'dB'],
    ['BRLS=',num2str(Meta_rls), 'dB    C-BRLS=',num2str(Meta_afrls), 'dB   Gain=',num2str( abs( Meta_afrls-Meta_rls ) ),'dB'],
    ['autol1-BRLS=',num2str(Meta_l1rls), 'dB    C-autol1-BRLS=',num2str(Meta_afl1rls), 'dB   Gain=',num2str( abs( Meta_afl1rls-Meta_l1rls ) ),'dB'],
    ['IPNLMS=',num2str(Meta_ipnlms), 'dB    C-IPNLMS=',num2str(Meta_afipnlms), 'dB   Gain=',num2str( abs( Meta_afipnlms-Meta_ipnlms ) ),'dB'],
    ['mNLMS=',num2str(Meta_mnlms), 'dB    C-mNLMS=',num2str(Meta_afmnlms), 'dB   Gain=',num2str( abs( Meta_afmnlms-Meta_mnlms ) ),'dB'],
    ['BSCG=',num2str(Meta_scg), 'dB    C-BSCG=',num2str(Meta_afscg), 'dB   Gain=',num2str( abs( Meta_afscg-Meta_scg ) ),'dB'],
    ['KBSCG=',num2str(Meta_kronscg), 'dB    C-KBSCG=',num2str(Meta_afkronscg), 'dB   Gain=',num2str(abs( Meta_afkronscg-Meta_kronscg ) ),'dB'],
    ['BCE=',num2str(Meta_C), 'dB    C-BCE=',num2str(Meta_afC), 'dB   Gain=',num2str(abs( Meta_afC-Meta_C ) ),'dB']})
%% Plot  Performance Measure MSD and Compute Gain
MSD_ls = 20*log10( vecnorm( h_ls - cali_h ) ); %./ vecnorm( cali_h ) );
MSD_rls = 20*log10( vecnorm( h_rls - cali_h ) ); %./ vecnorm( cali_h) );
MSD_l1rls = 20*log10( vecnorm( h_l1rls - cali_h ) ); %./ vecnorm( cali_h ) );
MSD_ipnlms = 20*log10( vecnorm( h_ipnlms - cali_h ) ); %./ vecnorm( cali_h ) );
MSD_mnlms = 20*log10( vecnorm( h_mnlms - cali_h ) ); %./ vecnorm( cali_h ) );
MSD_scg = 20*log10( vecnorm( h_scg - cali_h ) ); %./ vecnorm( cali_h ) );
MSD_kronscg = 20*log10( vecnorm( h_scgkron - cali_h ) ); %./ vecnorm( cali_h ) );
MSD_C = 20*log10( vecnorm( h_C - cali_h ) ); %./ vecnorm( cali_h ) );

MSD_afls = 20*log10( vecnorm( afh_ls - cali_h ) ); %./ vecnorm( cali_h ) );
MSD_afrls = 20*log10( vecnorm( afh_rls - cali_h ) ); %./ vecnorm( cali_h ) );
MSD_afl1rls = 20*log10( vecnorm( afh_l1rls - cali_h ) ); %./ vecnorm( cali_h ) );
MSD_afmnlms = 20*log10( vecnorm( afh_mnlms - cali_h ) ); %./ vecnorm( cali_h ) );
MSD_afipnlms = 20*log10( vecnorm( afh_ipnlms - cali_h ) ); %./ vecnorm( cali_h ) );
MSD_afscg = 20*log10( vecnorm( afh_scg - cali_h ) ); %./ vecnorm( cali_h ) );
MSD_afkronscg = 20*log10( vecnorm( afh_scgkron - cali_h ) ); %./ vecnorm( cali_h ) );
MSD_afC = 20*log10( vecnorm( afh_C - cali_h ) ); %./ vecnorm( cali_h ) );

figure;
plot(MSD_ls,'-o');  hold on;  plot(MSD_afls,'--o');  hold on;
plot(MSD_rls,'-+');  hold on;plot(MSD_afrls,'--+');  hold on;
plot(MSD_l1rls,'-x');  hold on;plot(MSD_afl1rls,'--x');  hold on;
plot(MSD_ipnlms,'-.');  hold on;plot(MSD_afipnlms,'--.');  hold on;
plot(MSD_mnlms,'-s');  hold on;plot(MSD_afmnlms,'--s');  hold on;
plot(MSD_scg,'-*');  hold on;plot(MSD_afscg,'--*');  hold on;
plot(MSD_kronscg,'-^');  hold on;plot(MSD_afkronscg,'--^');  hold on;
plot(MSD_C,'-d');  hold on;plot(MSD_afC,'--d');  hold on;
legend('LS','com-LS','BRLS','com-BRLS','l1BRLS','com-l1BRLS','IPNLMS','com-IPNLMS','mNLMS','com-mNLMS','BSCG','com-BSCG','KBSCG','com-KBSCG','BCE','com-BCE');
xlabel('Pulse');ylabel('MSD (dB)');title(['SNR=',num2str(SNR),'dB  MSD'])

EMSD_ls = mean(MSD_ls);        EMSD_afls = mean(MSD_afls);
EMSD_rls = mean(MSD_rls);        EMSD_afrls = mean(MSD_afrls);
EMSD_l1rls = mean(MSD_l1rls);        EMSD_afl1rls = mean(MSD_afl1rls);
EMSD_mnlms = mean(MSD_mnlms);        EMSD_afmnlms = mean(MSD_afmnlms);
EMSD_ipnlms = mean(MSD_ipnlms);        EMSD_afipnlms = mean(MSD_afipnlms);
EMSD_scg = mean(MSD_scg);        EMSD_afscg = mean(MSD_afscg);
EMSD_kronscg = mean(MSD_kronscg);        EMSD_afkronscg = mean(MSD_afkronscg);
EMSD_C = mean(MSD_C);        EMSD_afC = mean(MSD_afC);

disp(['Output follow MSD'])
disp({['LS=',num2str(EMSD_ls), 'dB    C-LS=',num2str(EMSD_afls), 'dB   Gain=',num2str( abs( EMSD_afls-EMSD_ls ) ),'dB'],
    ['BRLS=',num2str(EMSD_rls), 'dB    C-BRLS=',num2str(EMSD_afrls), 'dB   Gain=',num2str( abs( EMSD_afrls-EMSD_rls ) ),'dB'],
    ['l1BRLS=',num2str(EMSD_l1rls), 'dB    C-l1BRLS=',num2str(EMSD_afl1rls), 'dB   Gain=',num2str( abs( EMSD_afl1rls-EMSD_l1rls ) ),'dB'],
    ['IPNLMS=',num2str(EMSD_ipnlms), 'dB    C-IPNLMS=',num2str(EMSD_afipnlms), 'dB   Gain=',num2str( abs( EMSD_afipnlms-EMSD_ipnlms ) ),'dB'],
    ['mNLMS=',num2str(EMSD_mnlms), 'dB    C-mNLMS=',num2str(EMSD_afmnlms), 'dB   Gain=',num2str( abs( EMSD_afmnlms-EMSD_mnlms ) ),'dB'],
    ['BSCG=',num2str(EMSD_scg), 'dB    C-BSCG=',num2str(EMSD_afscg), 'dB   Gain=',num2str( abs( EMSD_afscg-EMSD_scg ) ),'dB'],
    ['KBSCG=',num2str(EMSD_kronscg), 'dB    C-KBSCG=',num2str(EMSD_afkronscg), 'dB   Gain=',num2str(abs( EMSD_afkronscg-EMSD_kronscg ) ),'dB'],
    ['BCE=',num2str(EMSD_C), 'dB    C-BCE=',num2str(EMSD_afC), 'dB   Gain=',num2str(abs( EMSD_afC-EMSD_C ) ),'dB']})
%% Plot hEst results
figure;
ind = [1:200];

subplot(4,4,1);imagesc( abs(h_ls(ind,:)) );xlabel('Pulse'); ylabel('delay');title('Drift CIRs  LS');                             %caxis([clow chigh]) 
subplot(4,4,2);imagesc( abs( afh_ls(ind,:)) ); xlabel('Pulse'); ylabel('delay');title('Com LS');          %caxis([clow chigh])

subplot(4,4,3);imagesc( abs(h_rls(ind,:)) ); xlabel('Pulse'); ylabel('delay');title('Drift CIRs  BRLS');                       %caxis([clow chigh])
subplot(4,4,4);imagesc( abs( afh_rls(ind,:)) );xlabel('Pulse'); ylabel('delay');title('Com BRLS');      %caxis([clow chigh])

subplot(4,4,5);imagesc( abs(h_l1rls(ind,:)) ); xlabel('Pulse'); ylabel('delay');title('Drift CIRs  autol1-BRLS');                       %caxis([clow chigh])
subplot(4,4,6);imagesc( abs( afh_l1rls(ind,:)) );xlabel('Pulse'); ylabel('delay');title('Com autol1-BRLS');      %caxis([clow chigh])

subplot(4,4,7);imagesc( abs(h_ipnlms(ind,:)) ); xlabel('Pulse'); ylabel('delay');title('Drift CIRs  IPNLMS');                       %caxis([clow chigh])
subplot(4,4,8);imagesc( abs( afh_ipnlms(ind,:)) );xlabel('Pulse'); ylabel('delay');title('Com IPNLMS');      %caxis([clow chigh])

subplot(4,4,9);imagesc( abs(h_mnlms(ind,:)) ); xlabel('Pulse'); ylabel('delay');title('Drift CIRs  mNLMS');                       %caxis([clow chigh])
subplot(4,4,10);imagesc( abs( afh_mnlms(ind,:)) );xlabel('Pulse'); ylabel('delay');title('Com mNLMS');      %caxis([clow chigh])

subplot(4,4,11);imagesc( abs(h_scg(ind,:)) );xlabel('Pulse'); ylabel('delay');title('Drift CIRs  BSCG');                        %caxis([clow chigh]) 
subplot(4,4,12);imagesc( abs( afh_scg(ind,:)) );xlabel('Pulse'); ylabel('delay');title('Com BSCG');     %caxis([clow chigh])

subplot(4,4,13);imagesc( abs(h_scgkron(ind,:)) );xlabel('Pulse'); ylabel('delay');title('Drift CIRs  KBSCG');                             %caxis([clow chigh]) 
subplot(4,4,14);imagesc( abs(afh_scgkron(ind,:)) );xlabel('Pulse'); ylabel('delay');title('Com KBSCG');                             %caxis([clow chigh]) 

subplot(4,4,15);imagesc( abs(h_scgkron(ind,:)) );xlabel('Pulse'); ylabel('delay');title('Drift CIRs  BCE');                             %caxis([clow chigh]) 
subplot(4,4,16);imagesc( abs(afh_scgkron(ind,:)) );xlabel('Pulse'); ylabel('delay');title('Com BCE');                             %caxis([clow chigh]) 


%---------------Continuous CIRs---------------%

figure;imagesc(abs(h(ind,:)));xlabel('Pulse');ylabel('delay'); title('Drift CIRs')
figure;imagesc(abs(cali_h(ind,:)));xlabel('Pulse');ylabel('delay'); title('Compensated CIRs')




%% Function Section
function USQ_2 = UnderScore2(X, h_2,m1, m2, P)
      
%      [m2,~] = size(h_2);
%      m1 = size(X,1) / m2;
ind = m1*m2;
     
     QP=[];
         for p=1:P
             Qp =  X(1:ind,:).'*kron(eye(m1),h_2(:,p)) ;
             QP = [QP,Qp];
         end
         USQ_2 = QP;
             
end

function USQ_1 = UnderScore1(X, h_1, m1, m2, P)
      
%      [m1,~] = size(h_1);
%      m2 = size(X,1) / m1;
ind = m1*m2;
     
     QP=[];
         for p=1:P
             Qp =  X(1:ind,:).'*kron(h_1(:,p),eye(m2)) ;
             QP = [QP,Qp];
         end
         USQ_1 = QP;
             
end

function C = kronSum( A,B,P )

% return the kronecker sum of two matrix

C=[];
for p = 1:P
    
    c = kron(A(:,p),B(:,p));
    C = [C,c];
    
end
     C = sum(C,2);
end