% Experiment Data Processing for removing natural CIR to detect weak moving target

clear all;  close all;  clc;

addpath('../functions','../data')              % Add this line to update the path

PN = 13; %PulseNumber
SRR = -2.5;

initialK  = 10; % target occur pulse   (ps: initialK>2)
K = PN-initialK+1;             

%% Measurment Section
load 'DATA0707'

N_t = BlockNoise_0707(:,1:PN);  % Pulses Noise
Y_t  = BlockData_0707(:,1:PN);     % Pulses Receive Data (no target)

[L,PulseNum] = size( Y_t );          % Used for processing DataLength and PulseNum 

%% basic parameters
      % transmit signal, regressor
      Fs = 48e3; fs = 16e3; 
      s = decimate(TransSig , Fs/fs);      %Transmit Signal
      M = L - length( s );                        %estimate vector dim
      X = toeplitz( [ s(1);zeros(M-1,1) ],[ s;zeros(M,1) ] );  % Toeplitz regressor

for mc = 1:1000
      
      
%% Add noise into real data
      % normlized, add noise, each pulse
      Y1 = Y_t ./ max( abs( Y_t ) );     %Normlized measure per pulse
      N1 = N_t ./ max( abs( N_t ) );   %Normlized noise per pulse 
      Y0 = Y1;                          %Original data, true values
      Y   = Y1 + 12* randn(L,1) * std( N1 );   %Add noise per pulse, simulated data

%% Add Target into real data  
      % uniform speed/delay, Endpulse occur target, zoom echo amp
      
%       dtau     = 3e-3;
%       targetS = formtargetS( Y, s, fs, dtau );                                                     %Target echo all pulses      
%       targetS=targetS(:,initialK:end);
    
      [targetS, TarResp] = formComplexTargetSignal ( Y, s, fs, 1, K );
      
      alpha    = std( Y0(:,initialK:end) ) ./ std( targetS ) * 10^( SRR/20 );                   %zoom echo amp ratio
     
Z0 = Y;
Z1 = Z0;

Z1(:,initialK:end)= Z1(:,initialK:end) + alpha.*targetS;  
%% Calibrate shifting receive data y Section
for t = 1:PulseNum
    [cali_Z0(:,t) , Est_tauZ0(t)] = DriftEstimator(Z0(:,1),Z0(:,t),fs);
    [cali_Z1(:,t) , Est_tauZ1(t)] = DriftEstimator(Z1(:,1),Z1(:,t),fs);
    [cali_Y0(:,t) , Est_tau(t)]     = DriftEstimator(Y0(:,1),Y0(:,t),fs);
end

%% Check1 SNR & SRR
YhatLs = EstLs(X,cali_Y0);
srr  = 20*log10( std(alpha.*targetS) ./ std(YhatLs (:,initialK:end)) )
snr = 20*log10( std(YhatLs) ./ std(cali_Z0-YhatLs) )
fprintf('**** SRR is %4.1fdB **** \n', mean(srr)); 
fprintf('**** SNR is %4.1fdB **** \n', mean(snr));

subplot(211)
plot( (1:L)/fs , [ cali_Z0(:,1) cali_Y0(:,1)]);legend('Add noise CaliZ0','Orignal CaliY0');
xlabel('Time(s)');ylabel('Amplitude');title(['SNR=',num2str(mean(snr)),' dB'])
subplot(212)
plot( (1:L)/fs , [  cali_Y0(:,1) alpha(1)*targetS(:,1)]);legend('Reverberation','target');
xlabel('Time(s)');ylabel('Amplitude');title(['SRR=',num2str(mean(srr)),' dB'])
%% Detection Section
T1 (mc,:)  = ExpDetectAllPulses( cali_Z1, X, initialK, s, cali_Y0 ) ;
T0 (mc,:)  = ExpDetectAllPulses( cali_Z0, X, initialK, s, cali_Y0 ) ;
fprintf('BRLS_T0  is %4.2f;    autol1-BRLS_T0  is %4.2f;    IPNLMS_T0  is %4.2f;    mNLMS_T0  is %4.2f;    BSCG_T0  is %4.2f;    KBSCG_T0 is %4.2f;    BCE_T0 is %4.2f;    MF_T0 is %4.2f; \n', T0(mc,:))
fprintf('BRLS_T1  is %4.2f;    autol1-BRLS_T1  is %4.2f;    IPNLMS_T1  is %4.2f;    mNLMS_T1  is %4.2f;    BSCG_T1  is %4.2f;    KBSCG_T1 is %4.2f;    BCE_T1 is %4.2f;    MF_T1 is %4.2f; \n', T1(mc,:))
fprintf('**** Monte Carlo is %d **** \n\n\n\n ',mc)

close all
end


%% Plot ROC 
[Pfa_rls,Pd_rls]  =  formROC( T0(:,1), T1(:,1) );
[Pfa_l1rls,Pd_l1rls]  =  formROC( T0(:,2), T1(:,2) );
[Pfa_ipn,Pd_ipn]  =  formROC( T0(:,3), T1(:,3) );
[Pfa_mnlms,Pd_mnlms]  =  formROC( T0(:,4), T1(:,4) );
[Pfa_scg,Pd_scg]  =  formROC( T0(:,5), T1(:,5) );
[Pfa_scgkron,Pd_scgkron]  =  formROC( T0(:,6), T1(:,6) );
[Pfa_C,Pd_C]  =  formROC( T0(:,7), T1(:,7) );

[Pfa_Mf,Pd_Mf]  =  formROC( T0(:,8), T1(:,8) );

figure

plot(Pfa_rls,Pd_rls,'--o'); hold on;
plot(Pfa_l1rls,Pd_l1rls,'--*'); hold on;
plot(Pfa_ipn,Pd_ipn,'-->'); hold on;
plot(Pfa_mnlms,Pd_mnlms,'--+'); hold on;
plot(Pfa_scg,Pd_scg,'--d'); hold on;
plot(Pfa_scgkron,Pd_scgkron,'--^');hold on;
plot(Pfa_C,Pd_C,'--s');hold on;


plot(Pfa_Mf,Pd_Mf);
line([0:1],[0:1],'color','black','Linewidth',1,'LineStyle','--'); % plot diagnose line
legend('BRLS','autol1-BRLS','IPNLMS','mNLMS','BSCG','KBSCG','BCE','UMF');
xlabel('P_f');ylabel('P_d');%title(['ROC SNR=',num2str(SNR),'dB'])



%% For paper Fig15
% 
open('Exp_xxx.fig');
lh = findall(gca, 'type', 'line');% ͼжߣlhΪһ
xc = get(lh, 'xdata');            % ȡxݣxcһԪ
yc = get(lh, 'ydata');            % ȡyݣycһԪ
%
figure;
plot(xc{9}, yc{9},'--o');hold on;%BRLS
plot(xc{8}, yc{8},'--*');hold on;%autol1-BRLS
plot(xc{7}, yc{7},'-->');hold on;%IPN
%
plot(xc{4}, yc{4},'--^');hold on;%KBSCG
plot(xc{3}, yc{3},'--+');hold on;%BCE
plot(xc{6}, yc{6},'--d');hold on;%mNLMS
plot(xc{5}, yc{5},'--s');hold on;%BSCG
%
plot( xc{2},yc{2} );hold on;%UMF
plot(xc{1}, yc{1},'--','linewidth',1);hold on;%diagline

legend('BRLS','autol1-BRLS','IPNLMS','mNLMS','BSCG','KBSCG','BCE','UMF');
% legend('mNLMS','BSCG','KBSCG','BCE');
xlabel('P_f');ylabel('P_d');