% Simulation Data Processing for removing natural CIR to detect weak moving target

clear all; close all; clc;

addpath('../functions','../data')              % Add this line to update the path

% Note when PulseNum = initialK calling CaseA, which means only use pulse
% with target, further when PulseNum > initialK calling CaseB, initialK:end
% are including the target, CaseA is special case of CaseB.
PulseNum = 8; %need change here to operate CaseA or CaseB
initialK = 8;  
K = PulseNum-initialK+1;                                         %added target pulseNmuber

SNR = -15;                %Signal Noise ratio
SRR = -3;                %Signal Reverberation ratio

c     = 1500;               %Sound speed
PRT = 0.25;                %Pulse Repeat Time


       %% transmit signal, CIR, 
       Tp = 0.2;   fs = 16e3;  f1 = 3e3;  f2 = 7e3;  
       Ts = 1/fs;
       s = chirp( (0:Ts:Tp-Ts).', f1, Tp, f2 );            %transmit signal
       
       L = PRT * fs;       %length per pulse
       N = length(s);     %length of transmit signal
       M = L - N;           %length of per CIR
       
       load 'P27_0707.mat';  
       z = [h0707; zeros(L-N-length(h0707),1)];     %CIR
       H = z * ones(1,PulseNum);                           %CIRs
       %% MC
for mc = 1:1000       
       
%% Measurement 
       % H0: no target pulses  /  H1: with target pulses
       for t = 1:PulseNum
           [Y(:,t) , X] = PAS( s, H(:,t), L, SNR );
       end
%% Add Target into Sim data
       % uniform speed/delay, Endpulse occur target, zoom echo amp
%        dtau = 3e-3;
%        targetS = formtargetS( Y, s, fs, dtau );            %Target echo all pulses
%        targetS=targetS(:,initialK:end);
       
       [targetS, TarResp] = formComplexTargetSignal ( Y, s, fs, 1, K );
       
           alpha = 10^(SRR/20) * std( X'*H(:,initialK:end)) ./ std(targetS);               % Zoom coefficient for targetecho amplitude                                                                             
Z0 = Y;
Z1 = Z0;

Z1(:,initialK:end)= Z1(:,initialK:end) + alpha.*targetS;
%% Check1 SNR & SRR      
srr  = 20*log10( std(alpha.*targetS) ./ std(X'*H(:,initialK:end)) );
snr = 20*log10( std( X'*H ) ./ std(Z0-X'*H) );           % desired SNR
fprintf('**** SRR is %4.1fdB **** \n', mean(srr)); 
fprintf('**** SNR is %4.1fdB **** \n',mean(snr));

% Plot simulated measurement and target signal (noise-free),
% Plot simulated measurement with moving target (noise),
% Stem complex target response

% subplot(311)
% stem( (1:M)/fs,TarResp(:,[K]) );
% xlabel('Time(s)'), ylabel('Amplitude')
subplot(312)
plot( (1:L)/fs, [X'*H(:,initialK) alpha(1)*targetS(:,1)] );
legend('X*h','targetSignal');title(['SRR=',num2str(SRR),'dB'])
xlabel('Time(s)'), ylabel('Amplitude')
subplot(313)
plot( (1:L)/fs, [ Z1(:,initialK) Z0(:,initialK) ] );
legend('Z1-with target','Z0-without target');title(['SNR =',num2str(SNR),'dB'])
xlabel('Time(s)'), ylabel('Amplitude')

%% Detection Section
T1 (mc,:) =  SimDetectAllPulses( Z1, X, initialK, s, H ); %with target 
T0 (mc,:) =  SimDetectAllPulses( Z0, X, initialK, s, H ); %without target 
fprintf('BRLS_T0  is %4.2f;    autol1-BRLS_T0  is %4.2f;    IPNLMS_T0  is %4.2f;     mNLMS_T0  is %4.2f;    BSCG_T0  is %4.2f;    KBSCG_T0 is %4.2f;     BCE_T0 is %4.2f;    MF_T0 is %4.2f; \n', T0(mc,:))
fprintf('BRLS_T1  is %4.2f;    autol1-BRLS_T1  is %4.2f;    IPNLMS_T1  is %4.2f;     mNLMS_T1  is %4.2f;    BSCG_T1  is %4.2f;    KBSCG_T1 is %4.2f;     BCE_T1 is %4.2f;    MF_T1 is %4.2f; \n', T1(mc,:))
fprintf('**** Monte Carlo is %d **** \n\n\n\n ',mc)

close all
end


%% Plot ROC 
[Pfa_rls,Pd_rls]  =  formROC( T0(:,1), T1(:,1) );
[Pfa_l1rls,Pd_l1rls]  =  formROC( T0(:,2), T1(:,2) );
[Pfa_ipn,Pd_ipn]  =  formROC( T0(:,3), T1(:,3) );
[Pfa_mnlms,Pd_mnlms]  =  formROC( T0(:,4), T1(:,4) );
[Pfa_scg,Pd_scg]  =  formROC( T0(:,5), T1(:,5) );
[Pfa_scgkron,Pd_scgkron]  =  formROC( T0(:,6), T1(:,6) );
[Pfa_C,Pd_C]  =  formROC( T0(:,7), T1(:,7) );

[Pfa_Mf,Pd_Mf]  =  formROC( T0(:,8), T1(:,8) );

figure

plot(Pfa_rls,Pd_rls,'--o'); hold on;
plot(Pfa_l1rls,Pd_l1rls,'--*'); hold on;
plot(Pfa_ipn,Pd_ipn,'-->'); hold on;
plot(Pfa_mnlms,Pd_mnlms,'--+'); hold on;
plot(Pfa_scg,Pd_scg,'--d'); hold on;
plot(Pfa_scgkron,Pd_scgkron,'--^');hold on;
plot(Pfa_C,Pd_C,'--s');hold on;


plot(Pfa_Mf,Pd_Mf);
line([0:1],[0:1],'color','black','Linewidth',1,'LineStyle','--'); % plot diagnose line
legend('BRLS','autol1-BRLS','IPNLMS','mNLMS','BSCG','KBSCG','BCE','UMF');
xlabel('P_f');ylabel('P_d');%title(['ROC SNR=',num2str(SNR),'dB'])
