% Here we just consider the effect of parameter \xi (sparse factor) on BSCG algorithm,
% and BRLS algorithm is compared
% we use 40 pulses sparse and  non-staitionary channel
% Only change sparse factor $\xi$

% Conclusion: 0dB BSCG algorithm obtain same the results as Paper 'A SparseConjugate Gradient Adaptive Filter, Bhaskar D. Rao2020';
% The $\xi$ smaller, the sparser , and in low SNR case,  sparse characteristic doesn't affect BSCG
% performance much;


clear all; close all;  clc;

addpath('../functions','../data');

SNR = 0 ;
Part = 1;% time-varying part
partnum = 80;% CIRs in one part, and all CIRs are time-invariant
PulseNum = partnum*Part;

%% Transmit Signal
Tp = 0.2;   fs = 16e3;  f1 = 3e3; f2 = 7e3;  
s = transmitLFM(Tp,fs,f1,f2);

%% Measurment Section
% load 'spah.mat';              % sparse CIRs
load 'P27_0707'; h=h0707; % non-sparse CIRs
% load 'Sparseh';   h = sh;

M = length(h)

w = envelope ( h );
for part = 1 : Part
    Delta_h =  w.*randn(M,1);
    for  t = partnum*(part-1)+1 : partnum*part
        h_t(:,t) = h + Delta_h;
        [ cali_y(:,t) , X ] = simulatePulseData_toeplitz(s,h_t(:,t),1,SNR);
    end
end

%%%%%%%%
% figure;
% plot(h_t(:,10)); hold on;
% plot(h_t(:,50));hold on
% legend('$h_{part1}$','$h_{part2}$','interpreter','latex')
%%%%%%%%%%

%% Estimatie Section  
lambda = 0.7;

 %% BRLS Settings
h_rls = zeros(M,PulseNum);
P_n = eye(M);
 %% BSCG Settings
 %% xi selection START

for i = 1:8

    xi= 1 - (i-1)*.1;    %sparse factor

delta = 1e-6;
c = 1e-4;

h_scg = zeros(M,PulseNum); 
Rn = eye(M) ; 
bn = zeros(M,1); 
pn = zeros(M,1); 

%% LOOP
for t = 1:PulseNum
     yb = cali_y(:,t);

     %% BRLS loop
     [h_rls(:,t+1),P_n] = RLSupdate(X, yb, h_rls(:,t), P_n, lambda);
     %% BSCG loop
      [h_scg(:,t+1),Rn,bn,pn] = SCGupdate(lambda, xi, delta, c, X, yb, h_scg(:,t), Rn, bn, pn);
      
       disp(['pulse=',num2str(t)])
end      
%% output hEst
h_rls = h_rls(:,2:end);
h_scg = h_scg(:,2:end);

%% Compute Performance Measure  eta
eta_rls  = 20*log10( vecnorm( X.'*h_rls - X.'*h_t ) ./ vecnorm( X.'*h_t ) );
eta_scg(:,i)= 20*log10( vecnorm( X.'*h_scg - X.'*h_t ) ./ vecnorm( X.'*h_t ) );

end

%% Plot effect of xi on sparsity
figure;

plot(eta_rls,'-o');  hold on;
plot(eta_scg,'--*');  hold on;
legend('BRLS','$BSCG  (\xi=1)$','$BSCG(\xi=0.9)$','$BSCG  (\xi=0.8)$','$BSCG  (\xi=0.7)$','$BSCG  (\xi=0.6)$','$BSCG  (\xi=0.5)$','$BSCG  (\xi=0.4)$','$BSCG  (\xi=0.3)$','interpreter','latex')
xlabel('Pulse Number');ylabel('\eta(dB)');
title(['\lambda=',num2str(lambda),'  SNR=',num2str(SNR),'dB']);



%% paper fig4
open('xi.fig');
lh = findall(gca, 'type', 'line');% ͼжߣlhΪһ
xc = get(lh, 'xdata');            % ȡxݣxcһԪ
yc = get(lh, 'ydata');            % ȡyݣycһԪ
%ȡõ2ߵxy
x2=xc{2};
y2=yc{2};
%%
figure;
plot(xc{1}, yc{9},'--o');hold on;%BRLS
plot(xc{1}, yc{8},'--.');hold on;%1
plot(xc{1}, yc{7},'--s');hold on;%0.9
plot(xc{1}, yc{5},'--*');hold on;%0.7
plot(xc{1}, yc{3},'--+');hold on;%0.5
plot(xc{1}, yc{1},'-->');hold on;%0.3
legend('BRLS','$BSCG  (\xi=1)$','$BSCG(\xi=0.9)$','$BSCG  (\xi=0.7)$','$BSCG  (\xi=0.5)$','$BSCG  (\xi=0.3)$','interpreter','latex')
xlabel('Pulse Number');ylabel('\eta(dB)');

