function [oh,   h_rlsn, P_n,...
    h_scgn, Rn, bn, pn,...
    h1_t, h2_t, Rn1, Rn2, bn1, bn2, pn1, pn2, Q_2,...
    h_ipn] = IPExpBCEupdate(lambda, xi, delta, c,...
                                                                        w_0, P_0,...
                                                                        h0, R0, b0, p0,...
                                                                        X, Q_2, y, y0,...
                                                                        h1, h2, R1, R2, b1, b2, p1, p2, lam1, lam2, m1, m2, P,...
                                                                        h_ipn0, mu, sigma)

% IPBCEupdate: Improved BCEupdate, exploit the IPNLMS the fast convergence
% advantage, using the switch case way to obtain optimal output.

% per pulse processing
% Output:
% the best of reults of BRLS, BSCG, IPNLMS, KBSCG algorithms

% Note that h is true values, so nor =  X.'*h; (i.e., nor is normlized y true values)

M = length(h0);
oh = zeros(M,1); %BCE estimate

%% Estimate hEst
[h_rlsn, P_n] = RLSupdate(X, y, w_0, P_0, lambda);   % BRLS
[h_scgn, Rn, bn, pn]  = SCGupdate(lambda, xi, delta, c, X, y, h0, R0, b0, p0);   % BSCG
[h_ipn] = IPNLMSupdate(X, y, mu, h_ipn0, delta, sigma);    %IPNLMS
[h_kscg, h1_t, h2_t, Rn1, Rn2, bn1, bn2, pn1, pn2, Q_2] = KronSCGupdate(X, Q_2, y, h1, h2, R1, R2, b1, b2, p1, p2,...
                                                                                                                      lam1, lam2, xi, delta, c, m1, m2, P);  % KBSCG
   

%% Compare eta=|desired-Est| / |desired|
       nor = y0;%X.' * h; %desired
       
       msdrls = 20*log10( norm( nor - X.'*h_rlsn ) ./ norm( nor ) );
       msdipn = 20*log10( norm( nor - X.'*h_ipn ) ./ norm( nor ) );
       msdscg = 20*log10( norm( nor - X.'*h_scgn ) ./ norm( nor ) );
       msdkscg = 20*log10( norm( nor - X.'*h_kscg ) ./ norm( nor ) );
       
       ind =[1: m1*m2];
       H1 = reshape(h1,[m1,P]);  H2= reshape(h2,[m2,P]);
       dmsdkscg = 20*log10( norm( nor - X.'*h_kscg ) ./ norm( nor ) ) - ...
                                20*log10( norm( nor - X(ind,:).'*kronSum( H1, H2, P ) ) ./ norm( nor ) );
       dmsdipn   =  20*log10( norm( nor - X.'*h_ipn ) ./ norm( nor ) ) - ...
                                 20*log10( norm( nor - X.'*h_ipn0 ) ./ norm( nor ) );  
       dmsdscg   =  20*log10( norm( nor - X.'*h_scgn ) ./ norm( nor ) ) - ...
                                 20*log10( norm( nor - X.'*h0 ) ./ norm( nor ) );  
       dmsdrls     =  20*log10( norm( nor - X.'*h_rlsn ) ./ norm( nor ) ) - ...
                                 20*log10( norm( nor - X.'*w_0) ./ norm( nor ) );
       
       %% select optimal as BCE output
       msdset = [msdkscg, msdipn, msdscg, msdrls];% order fixed, KBSCG,IPN,BSCG,BRLS
       dset = [dmsdkscg, dmsdipn, dmsdscg, dmsdrls];
%        disp({['msdKBSCG=',num2str(msdkscg),'dB'],['msdIPN=',num2str(msdipn),'dB']...
%          ['msdBSCG=',num2str(msdscg),'dB'],['msdBRLS=',num2str(msdrls),'dB']});
%        disp({['dKBSCG=',num2str(dmsdkscg),'dB'],['dIPN=',num2str(dmsdipn),'dB'],...
%          ['dBSCG=',num2str(dmsdscg),'dB'],['dBRLS=',num2str(dmsdrls),'dB']});     
       n = find(msdset==min(msdset)); %
       switch n
           case 1
               if n==1 && dset(n)<=0
                   oh = h_kscg;
%                    disp(['1,1'])
               elseif n==1 && dset(n)>0
                   oh = h_kscg;
                   h_ipn = h_kscg;
%                    disp(['1,2'])
               end
           case 2
               if n==2 && dset(n)<=0
                   oh = h_ipn;
%                    disp(['2,1'])
                elseif n==2 && dset(n)>0
                    oh = h_ipn;
                    h_scgn = h_ipn;
%                     disp(['2,2'])
               end
           case 3
                 if n==3 && dset(n)<=0
                   oh = h_scgn;
%                    disp(['3,1'])
                elseif n==3 && dset(n)>0
                    oh = h_scgn;
                    h_rlsn = h_scgn;
%                     disp(['3,2'])
                 end
             case 4
                 if n==4 && dset(n)<=0
                   oh = h_rlsn;
%                    disp(['4,1'])
                 else  
                    oh = h_rlsn;
                    h_ipn = h_rlsn;
                    h_scgn = h_rlsn;
%                     disp(['4,2'])
                 end
       end
      
end


%% Function Section
function USQ_2 = UnderScore2(X, h_2,m1, m2, P)
      
%      [m2,~] = size(h_2);
%      m1 = size(X,1) / m2;
ind = m1*m2;
     
     QP=[];
         for p=1:P
             Qp =  X(1:ind,:).'*kron(eye(m1),h_2(:,p)) ;
             QP = [QP,Qp];
         end
         USQ_2 = QP;
             
end

function USQ_1 = UnderScore1(X, h_1, m1, m2, P)
      
%      [m1,~] = size(h_1);
%      m2 = size(X,1) / m1;
ind = m1*m2;
     
     QP=[];
         for p=1:P
             Qp =  X(1:ind,:).'*kron(h_1(:,p),eye(m2)) ;
             QP = [QP,Qp];
         end
         USQ_1 = QP;
             
end

function C = kronSum( A,B,P )

% return the kronecker sum of two matrix

C=[];
for p = 1:P
    
    c = kron(A(:,p),B(:,p));
    C = [C,c];
    
end
     C = sum(C,2);
end